local AddonName, GGL = ...
--[[-----------------------------------------------------------------------------
Label Widget
Displays text and optionally an icon.
-------------------------------------------------------------------------------]]
local Type, Version = "GoldGoblinLootGUIWidget-ArchiveEntry", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, select, pairs = math.max, select, pairs
local tonumber = tonumber
local abs, floor = math.abs, math.floor
local format = string.format
local tbl_remove = table.remove
local string_len = string.len

local L = GGL.Locale

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent
local GetItemIcon, GetItemInfo = GetItemIcon, GetItemInfo
local BreakUpLargeNumbers = _G.BreakUpLargeNumbers

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]
local function ButtonSetText(self, text)
    self.buttonObj:SetText(text)
end

local function Button_OnClick(self, ...)
    if self.type == "delete" then
        self.obj:Fire("DeleteOnClick", ...)
    elseif self.type == "export" then
        if IsShiftKeyDown() then
            GGL.Export:FillExportWindow(GoldGoblinLootArchiveDB.stores.ReadOnly[self.obj.sessionID].data, "text")
        elseif self.obj.sessionInfo and self.obj.sessionInfo.exportData then
            GGL.Export:FillExportWindow(GGL.GetDecompressedString(self.obj.sessionInfo.exportData), "csv")
        end

        self.obj:Fire("ExportOnClick", ...)
    end
end

local function NewButton(widget, content, buttonType)
    local buttonObj = AceGUI:Create("Button")
    buttonObj:SetAutoWidth(true)
    buttonObj:SetCallback("OnClick", Button_OnClick)

    local button = buttonObj.frame
    button:ClearAllPoints()
    button:SetParent(content)
    button:Show()

    button._SetText = button.SetText
    button.SetText = ButtonSetText
    button.buttonObj = buttonObj
    buttonObj.type = buttonType
    buttonObj.obj = widget

    return button
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
    ["OnAcquire"] = function(self)
        self:SetArchiveData()
        self.exportButton:Hide()
    end,

    ["OnRelease"] = function(self)
        self.frame:Hide()
        self.exportButton:Hide()
	end,

    ["OnWidthSet"] = function(self, width)

    end,

    ["GetSortValue"] = function(self)
        return self.sortValue or 0
    end,

    ["SetArchiveData"] = function(self, archiveData, sessionID, sessionInfo)
        self.archiveData = archiveData
        self.sessionID = sessionID
        self.sessionInfo = sessionInfo
        if archiveData and sessionID and sessionInfo then
            self.textSessionID:SetText(format(L["|cffffd70aID:|r %s |cffffd70aWoW-Version:|r %s"], sessionID, sessionInfo.gameVersion or ""))
            self.textSize:SetText(format(L["|cff33ff99Size:|r %s"], BreakUpLargeNumbers(string_len(GoldGoblinLootArchiveDB.stores.ReadOnly[sessionID].data or ""))))
            if sessionInfo.startTime and sessionInfo.startTime > 0 then
                self.textStartTime:SetText(format(L["|cff33ff99Start:|r %s"], GGL.FormatTimeStamp(sessionInfo.startTime)))
            end
            if sessionInfo.endTime and sessionInfo.endTime > 0 then
                self.textEndTime:SetText(format(L["|cff33ff99End:|r %s"], GGL.FormatTimeStamp(sessionInfo.endTime)))
            end
            -- set sort value
            if sessionInfo.startTime and sessionInfo.startTime > 0 then
                self.sortValue = sessionInfo.startTime
            else
                self.sortValue = tonumber(string.match(sessionID, "%a(%d+)"))
            end
            self.exportButton:Show()
        else
            self.textSessionID:SetText("")
            self.textStartTime:SetText("")
            self.textEndTime:SetText("")
        end
    end,
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
    local widget = {}
    widget.type  = Type

    local frame = GGL.GUI:CreateBorderedFrame(UIParent)
    frame:SetHeight(40)
    widget.frame = frame

    local content = CreateFrame("Frame", nil, frame)
    content:SetPoint("TOPLEFT", frame, 5, -5)
    content:SetPoint("BOTTOMRIGHT", frame, -5, 5)
    content.obj = widget
    widget.content = content

    local textSessionID = content:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")
    textSessionID:SetPoint("TOPLEFT", content, "TOPLEFT", 2, -2)
    textSessionID:SetJustifyH("LEFT")
    textSessionID:SetJustifyV("MIDDLE")
    textSessionID:SetWordWrap(false)
    widget.textSessionID = textSessionID

    local textSize = content:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    textSize:SetPoint("TOPLEFT", textSessionID, "BOTTOMLEFT", 0, -2)
    textSize:SetJustifyH("LEFT")
    textSize:SetJustifyV("MIDDLE")
    textSize:SetWordWrap(false)
    widget.textSize = textSize

    local textStartTime = content:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    textStartTime:SetPoint("LEFT", textSize, "RIGHT", 10, 0)
    textStartTime:SetJustifyH("LEFT")
    textStartTime:SetJustifyV("MIDDLE")
    textStartTime:SetWordWrap(false)
    widget.textStartTime = textStartTime

    local textEndTime = content:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    textEndTime:SetPoint("LEFT", textStartTime, "RIGHT", 10, 0)
    textEndTime:SetJustifyH("LEFT")
    textEndTime:SetJustifyV("MIDDLE")
    textEndTime:SetWordWrap(false)
    widget.textEndTime = textEndTime

    local deleteButton = NewButton(widget, content, "delete")
    deleteButton:SetPoint("RIGHT", content, "RIGHT", -5, 0)
    deleteButton:SetText(DELETE)
    widget.deleteButton = textEndTime

    local exportButton = NewButton(widget, content, "export")
    exportButton:SetPoint("RIGHT", deleteButton, "LEFT", -2, 0)
    exportButton:SetText(L["Export"])
    widget.exportButton = exportButton

    -- methodes
    for method, func in pairs(methods) do
        widget[method] = func
    end

    return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)